#ifndef __INCLUDE_JM_GUI_BI__
#define __INCLUDE_JM_GUI_BI__

' -------------------------------------------------------------------
' BASE UI TYPES
' -------------------------------------------------------------------

Type UI_POINT_T
  x As Integer
  y As Integer
End Type

Type UI_RECT_T
  x As Integer
  y As Integer
  w As Integer
  h As Integer
End Type

Type UI_MSG_DATA_T
  mX As Integer
  mY As Integer
  mW AS Integer
  mB As Integer
  ky As Integer
End Type

Type UI_BOX_PTR As UI_BOX_T Ptr

#define BOXPROC_FUNC_T Function(ByVal this_ As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer

Type UI_BOX_T
  Owner As UI_BOX_PTR
  Prev As UI_BOX_PTR
  Next As UI_BOX_PTR
  Child As UI_BOX_PTR
  BoxProc As BOXPROC_FUNC_T
  DefProc As BOXPROC_FUNC_T

  Id As Integer
  rec As UI_RECT_T
  BoxType As Integer

  Flags As Integer            '' DOWN flag could be extradata
	Style As Integer            '' Frame style flags
  ''ForeColor As Integer        '' could be extra data
  ''BackColor As Integer        '' could be extra data
  Caption As Any Ptr          '' could be extra data

  ExtraDataSize As Integer    '' This must be the last member!!!

End Type

' -------------------------------------------------------------------
' UI ELEMENT MESSAGES
' -------------------------------------------------------------------

Enum UI_MESSAGES
  UI_MSG_NONE = 0
  UI_MSG_CREATE
  UI_MSG_DESTROY
	UI_MSG_MOVE
	UI_MSG_SIZE
  UI_MSG_PAINT
  UI_MSG_KEYPRESS
''  UI_MSG_GOTFOCUS
''  UI_MSG_LOSTFOCUS
  UI_MSG_MOUSE_DOWN
  UI_MSG_MOUSE_UP
  UI_MSG_MOUSE_MOVE
''  UI_MSG_MOUSE_OVER
''  UI_MSG_MOUSE_ENTER
''  UI_MSG_MOUSE_LEAVE
  UI_MSG_MOUSE_CLICK
	
	UI_MSG_CHANGE

End Enum

' -------------------------------------------------------------------
' UI ELEMENT ORDER TYPES
' -------------------------------------------------------------------

Enum UI_ORDER_TYPES
  UI_ORDER_BOTTOM = -2
  UI_ORDER_DOWN = -1
  UI_ORDER_NO_CHANGE = 0
  UI_ORDER_UP = 1
  UI_ORDER_TOP = 2
End Enum

' -------------------------------------------------------------------
' UI ELEMENT FLAGS
' -------------------------------------------------------------------

Const UI_BOX_FLAG_VISIBLE As Integer  = &h1
Const UI_BOX_FLAG_DISABLED As Integer = &h2
Const UI_BOX_FLAG_DOWN As Integer     = &h4
Const UI_BOX_FLAG_ROOT As Integer     = &h8000000
Const UI_BOX_FLAG_DRAG                = &H8

Const UI_BOX_FLAG_NORMAL = UI_BOX_FLAG_VISIBLE

#ifndef FALSE
Const False = 0, True = Not False, NULL = 0
#endif

' -------------------------------------------------------------------
' RECT FUNTIONS
' -------------------------------------------------------------------
Declare Function UI_SetRect( byval rec As UI_RECT_T Ptr, byval x As Integer, byval y As Integer, byval w As Integer, byval h As integer ) As Integer
Declare Function UI_SetRectEmpty( byval rec As UI_RECT_T Ptr ) As Integer
Declare Function UI_CopyRect( byval dst As UI_RECT_T Ptr, byval src As UI_RECT_T Ptr ) As Integer
Declare Function UI_OffsetRect( byval rec As UI_RECT_T Ptr, byval dx As Integer, byval dy As Integer ) As Integer
Declare Function UI_PositionRect( byval rec As UI_RECT_T Ptr, byval x As Integer, byval y As Integer ) As Integer
Declare Function UI_ResizeRect( byval rec As UI_RECT_T Ptr, byval w As Integer, byval h As Integer ) As Integer
Declare Function UI_SizeRect( byval rec As UI_RECT_T Ptr ) As Integer
Declare Function UI_InflateRect( byval rec As UI_RECT_T Ptr, byval dx As Integer, byval dy As Integer ) As Integer
Declare Function UI_IsPointInRect (byval rec AS UI_RECT_T Ptr, byval x AS Integer, byval y AS Integer) as Integer

' -------------------------------------------------------------------
' UI MANAGER
' -------------------------------------------------------------------

Declare Sub UI_Init()
Declare Sub UI_Close()

Declare Function UI_CreateBox(byval ownerbox As UI_BOX_PTR, _
  byval boxType As Integer, byval id As Integer, byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, _
  ByVal DefProc As BOXPROC_FUNC_T, _
  ByVal extraData As Integer, ByVal flags As Integer ) As UI_BOX_PTR

Declare Function UI_DeleteBox(byval box As UI_BOX_PTR) As Integer

Declare Function UI_ShowBox(byval box As UI_BOX_PTR) As Integer
Declare Function UI_HideBox(byval box As UI_BOX_PTR) As Integer
Declare Function UI_EnableBox(byval box As UI_BOX_PTR) As Integer
Declare Function UI_DisableBox(byval box As UI_BOX_PTR) As Integer

Declare Function UI_SetBoxOrder(byval box As UI_BOX_PTR, byval Order As Integer) As Integer
Declare Function UI_GetBoxDataPtr(byval box As UI_BOX_PTR) As Any Ptr

Declare Function UI_GetParentBox(byval box As UI_BOX_PTR) As UI_BOX_PTR
Declare Function UI_GetBoxFromID(byval box As UI_BOX_PTR, ByVal id As Integer) As UI_BOX_PTR

' -------------------------------------------------------------------
' UI HELPERS
' -------------------------------------------------------------------

Declare Function UI_PositionBox(byval box As UI_BOX_PTR, byval x As integer, byval y As integer) As Integer
Declare Function UI_MoveBox(byval box As UI_BOX_PTR, byval x As integer, byval y As integer, byval w As integer, byval h As integer) As Integer
Declare Function UI_ResizeBox(byval box As UI_BOX_PTR, byval w As integer, byval h As integer) As Integer

Declare Sub UI_SetCaption(byval box As UI_BOX_PTR, s As string)
Declare Function UI_GetCaption(byval box As UI_BOX_PTR) As String

Declare Function UI_CallDefaultProc(ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
Declare Function UI_CallBoxProc(ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer

Declare Function UI_Setfocus(byval box As UI_BOX_PTR) As UI_BOX_PTR
Declare Function UI_GetFocus() As UI_BOX_PTR

' -------------------------------------------------------------------
' BOX COORDS
' -------------------------------------------------------------------

Declare Function UI_GetScreenCoords(byval box As UI_BOX_PTR, byval rec As UI_RECT_T Ptr) As Integer
Declare Function UI_GetParentCoords(byval box As UI_BOX_PTR, byval rec As UI_RECT_T Ptr) As Integer
Declare Function UI_GetBoxCoords(byval box As UI_BOX_PTR, byval rec As UI_RECT_T Ptr) As Integer
Declare Function UI_GetBoxFromXY(byval box As UI_BOX_PTR, byval x AS Integer, byval y AS Integer) As UI_BOX_PTR

' -------------------------------------------------------------------
' UI INPUT
' -------------------------------------------------------------------

Declare Function UI_GetMouseCoords(ByRef x As Integer, ByRef y As Integer) As Integer
Declare Function UI_UpdateMouseInfo(byval box As UI_BOX_PTR, byval mX As Integer, byval mY As Integer, byval mW AS Integer, byval mB As Integer) As Integer
Declare Function UI_UpdateKeyInfo(byval box As UI_BOX_PTR, byval ky As Integer) As Integer
Declare Function UI_GenerateEvent() As Integer

Declare Function UI_SetCapture(byval box As UI_BOX_PTR) As UI_BOX_PTR
Declare Function UI_ReleaseCapture() As Integer
Declare Function UI_GetCapture() As UI_BOX_PTR

Declare Function UI_BeginDrag(byval box As UI_BOX_PTR, ByVal x As Integer, ByVal y As Integer) As UI_BOX_PTR
Declare Function UI_EndDrag(byval box As UI_BOX_PTR, ByVal x As Integer, ByVal y As Integer) As Integer
Declare Function UI_Drag(byval box As UI_BOX_PTR, ByVal x As Integer, ByVal y As Integer) As UI_BOX_PTR

' -------------------------------------------------------------------
' UI DRAWING 
' -------------------------------------------------------------------

Declare Function UI_Draw(byval box As UI_BOX_PTR, flags As Integer) As Integer

#endif
