''  jm_line_buffer.bas - jm_gui
''	Copyright (C) 2005,2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

Option Explicit

#include "jm_line_buffer.bi"

function LINE_BUFFER_Create (byval startsize as integer) as LINE_BUFFER_T Ptr
	dim buf as LINE_BUFFER_T Ptr
	buf = Allocate(sizeof(LINE_BUFFER_T))
	if (buf) then
		buf->msize = ((startsize + 1) + 3) and (not 3)
		buf->mindex = 0
		buf->mlength = 0
		buf->mdata = Allocate(buf->msize)
		buf->mdata[0] = 0
		buf->mflags = 0
		'' FIXME: on fail-clean-up and return null
	end if
	function = buf
end function

sub LINE_BUFFER_Destroy(ByVal buf as LINE_BUFFER_T Ptr)
	if (buf) then
		if buf->mdata then
			deallocate buf->mdata
		end if
		deallocate buf
	end if
end sub

sub LINE_BUFFER_Clear(byval buf as LINE_BUFFER_T Ptr)
	if (buf) then
		buf->mindex = 0
		buf->mlength = 0
		buf->mdata[0] = 0
		buf->mflags or= LINE_BUFFER_FLAG_CHANGED
	end if	
end sub

function LINE_BUFFER_GetBufferSize(byval buf as LINE_BUFFER_T ptr) As integer
  function = buf->msize
End function

sub LINE_BUFFER_SetBufferSize(byval buf as LINE_BUFFER_T ptr, byval size As integer)
	If size <> buf->msize and size > 1 Then
		buf->msize = ((size + 1) + 3) and (not 3)
		buf->mdata = Reallocate(buf->mdata, buf->msize)
		if buf->mlength >= buf->msize then	buf->mlength = buf->msize - 1
		if buf->mindex > buf->mlength then buf->mindex = buf->mlength
		buf->mdata[buf->mlength] = 0
	End If
End sub

function LINE_BUFFER_GetLength(byval buf as LINE_BUFFER_T Ptr) as integer
	if (buf) then
		function = buf->mlength
	end if
end function

function LINE_BUFFER_GetIndex(ByVal buf as LINE_BUFFER_T Ptr) as integer
	if (buf) then
		function = buf->mindex
	end if
end function

function LINE_BUFFER_SetIndex(ByVal buf as LINE_BUFFER_T Ptr, byval index as integer)
	function = -1
	if (buf) then
		if index < 0 then
			buf->mindex = 0
		elseif index > buf->mlength then
			buf->mindex = buf->mlength
		else
			buf->mindex = index
		end if
		function = index
	end if
end function

sub LINE_BUFFER_InsertChar(ByVal buf as LINE_BUFFER_T Ptr, byval ch as integer)
	dim i as integer
	if (buf) then
		if (ch) then
			if buf->mlength < buf->msize - 1 then
				if buf->mindex < buf->mlength then
					'' FIXME: use memmove()
					for i = buf->mlength + 1 to buf->mindex + 1 step - 1
						buf->mdata[i] = buf->mdata[i - 1]
					next i
				end if
				buf->mdata[buf->mindex] = (ch and 255)
				buf->mindex += 1
				buf->mlength += 1
				buf->mdata[buf->mlength] = 0
				buf->mflags or= LINE_BUFFER_FLAG_CHANGED
			end if
		end if
	end if										
end sub

sub LINE_BUFFER_BackChar(ByVal buf as LINE_BUFFER_T Ptr)
	dim i as integer
	if (buf) then
		if buf->mindex > 0 then
			'' FIXME: use memmove()
			for i = buf->mindex to buf->mlength
				buf->mdata[i - 1] = buf->mdata[i]
			next i
			buf->mindex -= 1
			buf->mlength -= 1
			buf->mflags or= LINE_BUFFER_FLAG_CHANGED
		end if
	end if
end sub

Sub LINE_BUFFER_DeleteChar(ByVal buf as LINE_BUFFER_T Ptr)
	dim i as integer
	if (buf) then
		if buf->mindex < buf->mlength then
			'' FIXME: use memmove()
			for i = buf->mindex to buf->mlength - 1
				buf->mdata[i] = buf->mdata[i + 1]
			next i
			buf->mlength -= 1
			buf->mflags or= LINE_BUFFER_FLAG_CHANGED
		end if
	end if
End Sub

function LINE_BUFFER_GetChar(byval buf as LINE_BUFFER_T Ptr, byval Index As integer) As Integer
	function = -1
	if (buf) then
		if Index >= 0 and Index < buf->mlength then
			function = buf->mdata[index]
		end if
	end if
end function

sub LINE_BUFFER_SetChar(byval buf as LINE_BUFFER_T Ptr, byval Index As integer, byval ch As Integer)
	if (buf) then
		if Index >= 0 and Index < buf->mlength then
			buf->mdata[index] = ch
		end if
	end if
end sub

function LINE_BUFFER_GetText(ByVal buf as LINE_BUFFER_T Ptr) as string
	dim txt as string, i as integer
	if (buf) then
		txt = string(buf->mlength,0)
		'' FIXME: use memcopy()
		for i = 0 to buf->mlength - 1
			txt[i] = buf->mdata[i]
		next i
	end if
	function = txt
end function

sub LINE_BUFFER_SetText(ByVal buf as LINE_BUFFER_T Ptr, txt as String)
	dim n as integer = len(txt)
	dim i as integer
	if n > buf->msize - 1 then
		n = buf->msize - 1
	end if
	'' FIXME: use memcopy()
	for i = 0 to n - 1
		buf->mdata[i] = txt[i]
	next i
	buf->mdata[n] = 0
	buf->mlength = n
	if buf->mindex > n then
		buf->mindex = n
	end if
	buf->mflags or= LINE_BUFFER_FLAG_CHANGED
end sub

sub LINE_BUFFER_ClearChanged(ByVal buf as LINE_BUFFER_T Ptr)
	buf->mflags and= (NOT LINE_BUFFER_FLAG_CHANGED)
end sub

function LINE_BUFFER_IsChanged(ByVal buf as LINE_BUFFER_T Ptr) as integer
	function = (buf->mflags and LINE_BUFFER_FLAG_CHANGED)
	buf->mflags and= (NOT LINE_BUFFER_FLAG_CHANGED)
end function

